// Word Automation Class in C++ 
//
// Author : Poonam Bajaj
//
// Date   : 24th December, 2000
//
// Header file : AutoWord.h

#include <ole2.h>
#include <comdef.h>

#include "AutoWord.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAutoWord::CAutoWord()
{
	m_pDispApp = NULL;
	m_pDocuments = NULL;

    //Initialize the COM libraries
    ::CoInitialize(NULL);
}

CAutoWord::~CAutoWord()
{
	Destroy();

	// Uninitialize com libraries
	CoUninitialize();
}

// Sets the file path as specified by the caller.
int CAutoWord::InitAutomation()
{
	// Get the CLSID for Word's Application Object
	CLSID clsid;
	CLSIDFromProgID(L"Word.Application", &clsid);  

	// Create an instance of the Word application and obtain the pointer
	// to the application's IUnknown interface
	IUnknown* pUnk;
	HRESULT hr = ::CoCreateInstance( clsid, NULL, CLSCTX_LOCAL_SERVER, IID_IUnknown, (void**) &pUnk);
	if (FAILED(hr))
	{
		OutputDebugString("Error in creating Word application instance\n");
		Destroy();
		return -1;
	}

	// Query IUnknown to retrieve a pointer to the IDispatch interface
	hr = pUnk->QueryInterface(IID_IDispatch, (void**)&m_pDispApp);

	// Get pointer to Documents interface.
	DISPPARAMS dp = { NULL, NULL, 0, 0 };  
	DISPID dispID;
	LPOLESTR szDoc = L"Documents";   
	IDispatch* pDocuments = NULL;
	VARIANT varRetVal;
	hr = m_pDispApp->GetIDsOfNames(IID_NULL, &szDoc, 1, LOCALE_SYSTEM_DEFAULT, &dispID);
	hr = m_pDispApp->Invoke(dispID, IID_NULL, LOCALE_SYSTEM_DEFAULT, DISPATCH_PROPERTYGET, &dp, &varRetVal, NULL, NULL);
	if (!SUCCEEDED(hr))
	{
		Destroy();		
		return -1;
	}
	
	// keep the Dispatch pointer in the member variable for later use.
	m_pDocuments = varRetVal.pdispVal;

	return 0;
}

// Prints the WORD document if everything goes fine
// otherwise error is returned
int CAutoWord::PrintDocument(char *strFilePath)
{
	// Open the document
	VARIANT varRetVal;
	EXCEPINFO excepInfo; 	// this variable contains exception info if any Invoke call fails
	VARIANTARG varg;
	varg.vt = VT_BSTR;
	varg.bstrVal = _bstr_t(strFilePath); // this is the MS-word document filename, must be changed to a valid filename that exists on disk
	DISPPARAMS dpOpen = { &varg, NULL, 1, 0 };
	DISPID dispOpenID;
	LPOLESTR szOpenDoc = L"Open";
	HRESULT hr = m_pDocuments->GetIDsOfNames(IID_NULL, &szOpenDoc, 1, LOCALE_SYSTEM_DEFAULT, &dispOpenID);
	hr = m_pDocuments->Invoke(dispOpenID, IID_NULL, LOCALE_SYSTEM_DEFAULT, DISPATCH_METHOD, &dpOpen, &varRetVal, &excepInfo, NULL);
	if (FAILED(hr))
	{
		OutputDebugString("Error opening the document\n");
		Destroy();
		return -1;
	}
	
	IDispatch* pDocument = varRetVal.pdispVal;

	// Call PrintOut method of the opened Document 
	DISPPARAMS dpPrint = { NULL, NULL, 0, 0 };
	DISPID dispPrintID;
	LPOLESTR szPrintDoc = L"PrintOut";
	hr = pDocument->GetIDsOfNames(IID_NULL, &szPrintDoc, 1, LOCALE_SYSTEM_DEFAULT, &dispPrintID);
	hr = pDocument->Invoke(dispPrintID, IID_NULL, LOCALE_SYSTEM_DEFAULT, DISPATCH_METHOD, &dpPrint, &varRetVal, NULL, NULL);
	if (FAILED(hr))
	{
		OutputDebugString("The document could not be printed\n");
		Destroy();
		return -1;
	}

	// Close the document now.
	DISPPARAMS dpClose = { NULL, NULL, 0, 0 };
	DISPID dispCloseID;
	LPOLESTR szCloseDoc = L"Close";
	hr = pDocument->GetIDsOfNames(IID_NULL, &szCloseDoc, 1, LOCALE_SYSTEM_DEFAULT, &dispCloseID);
	hr = pDocument->Invoke(dispCloseID, IID_NULL, LOCALE_SYSTEM_DEFAULT, DISPATCH_METHOD, &dpClose, &varRetVal, &excepInfo, NULL);
	
	return 0;
}

void CAutoWord::Destroy()
{
	// If just a fake call, return
	if (m_pDispApp == NULL)
		return;

	VARIANT varRetVal;
	EXCEPINFO excepInfo; 	// this variable contains exception info if any Invoke call fails
	
	// Variables to store 'Quit' related parameters
	DISPPARAMS dpQuit = { NULL, NULL, 0, 0 };
	DISPID dispQuit;
	LPOLESTR szQuit = L"Quit";

	//Quit the App
	HRESULT hr = m_pDispApp->GetIDsOfNames(IID_NULL, &szQuit, 1, LOCALE_SYSTEM_DEFAULT, &dispQuit);
	hr = m_pDispApp->Invoke(dispQuit, IID_NULL, LOCALE_SYSTEM_DEFAULT, DISPATCH_METHOD, &dpQuit, &varRetVal, &excepInfo, NULL);
	if (FAILED(hr))
	{
		OutputDebugString("Error in quitting WinWord.exe\n");
	}

	m_pDispApp = NULL;
}
